#! perl -w
#
# DISCLAIMER
# THE MATERIAL AND/OR SOFTWARE IS PROVIDED "AS IS".  RATIONAL AND ITS THIRD PARTY 
# LICENSORS DISCLAIM ALL REPRESENTATIONS OR WARRANTIES EXPRESS OR IMPLIED INCLUDING, 
# WITHOUT LIMITATION, THE WARRANTY OF MERCHANTABILITY, NON-INFRINGEMENT, TITLE OR 
# FITNESS FOR A PARTICULAR PURPOSE OR ARISING OUT OF THE COURSE OF DEALING, USAGE OR 
# TRADE PRACTICE, CONTENT OF THE MATERIAL OR SOFTWARE.  RATIONAL MAKES NO WARRANTIES 
# OR REPRESENTATIONS REGARDING THE ACCURACY OR COMPLETENESS OF THE MATERIAL AND/OR 
# SOFTWARE PROVIDED OR THAT IT WILL MEET LICENSEE'S REQUIREMENTS OR THAT THE MATERIAL 
# AND/OR SOFTWARE WILL BE ERROR FREE.  IN NO EVENT SHALL RATIONAL OR ITS LICENSORS BE 
# LIABLE TO LICENSEE OR A THIRD PARTY FOR ANY INDIRECT, DIRECT, NEGLIGENCE, SPECIAL, 
# OR CONSEQUENTIAL DAMAGES INCLUDING LOST PROFITS, LOST DATA AND THE LIKE ARISING OUT 
# OF OR IN CONNECTION WITH THIS RECEIPT OF MATERIAL AND/OR SOFTWARE EVEN IF RATIONAL 
# HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.  
#
$pid = 0;
$tsec = 10;
$doTruss = 0;
$outfile = "";
$version = "07-Mar-2002";
#
#-----------------------------------------------------------------------------
#
sub printHelp
{
    print <<EOT;

RTREPORT will run TOP and PSTACK on the RoseRT process, and also save environmental
variables and system information including patch information. Additionally, it will
run TRUSS on the RoseRT process if the truss (-t) option is enabled. Note that if you
are attempting to analyze a runtime problem that the output of pstack and truss are
most meaningful if run while the problem is occurring.


Syntax: rtperl $0 -o outfile [-p procid] [-t [secs]]

Args:   -o outfile   <outfile> with extensions of .env, .top, .ps (pstack),
                      and optionally .tr (truss) will be created.
        [-p procid]  Where <procid> is for RoseRT process that pstack and 
                     truss will be run on. Otherwise rtreport will use ps
                     in order to attempt to determine the RoseRT procid.
        [-t [secs]]  Run truss command on the RoseRT process
                     Kill truss process after specified seconds
                     If <secs> not supplied truss will be run for $tsec seconds

Sample usage:
   rtperl rtreport.pl                          # show this help info for rtreport.pl
   rtperl rtreport.pl -o my_report             # generate report without including truss
   rtperl rtreport.pl -o my_report -t          # include running truss $tsec seconds
   rtperl rtreport.pl -o my_report -t 5        # include running truss 5 seconds
   rtperl rtreport.pl -o my_report -p 12386    # RoseRT procid is 12386

Current version: $version

EOT
#
	exit 0;
}
#
#-----------------------------------------------------------------------------
#
sub readArgs
{
    &printHelp unless( scalar @ARGV );

    while( $_ = shift @ARGV )
    {
	if(    m[^-o$]  ) { $outfile = shift @ARGV; }
	elsif( m[^-p$]  ) { $pid  = shift @ARGV; }
	elsif( m[^-t$]  ) { $doTruss = 1; 
	                    if ($ARGV[0] && $ARGV[0] =~ /^\d+/) { $tsec = shift @ARGV; }
			  }
	else              { die "Unknown argument '$_'\n"; }
    }
}
#
#-----------------------------------------------------------------------------
#
sub main
{
	my ($tty, $time, $cmd, $tmpid);
	$tty = $time = $cmd = $tmpid = "";
#
    $| = 1;  # flush output after every print
    &readArgs;   # process the command line
	unless ($outfile) { &printHelp; }  # print help info and exit
#
# print the env vars ...
	open (OUTFILE, ">$outfile.env") or die ("could not open $outfile\n");
	print OUTFILE <<"EOF";
Environmental variables
============================================================
EOF
	foreach $key (sort keys %ENV) {
		print OUTFILE "$key=$ENV{$key}\n";
	}
	print OUTFILE "============================================================\n\n\n\n";
#
# append system and patch info ...
	@show_info = `showrev -a | sort`;
	print OUTFILE <<"EOF";
System and Patch Information  (from `showrev -a`)
============================================================
EOF
	$graphics = `ls /dev/fbs/\*`;
	$graphics =~ s/\/dev\/fbs\///;
	print OUTFILE "your graphics card type is: $graphics\n";
	
	foreach $line (@show_info) {
		print OUTFILE $line;
	}
	print OUTFILE "============================================================\n";
	close(OUTFILE);
#
# If procid not supplied - try to get it using ps ...
	unless ($pid) {
		@processes = `ps -a`;
		foreach $process (@processes) {
			$process =~ s/^\s+//;
			($tmpid, $tty, $time, $cmd) = split(/\s+/, $process);
			if ($cmd eq "RoseRT" && $time ne "0:00") { $pid = $tmpid; }
		}
	}
#
# Gather output of pstack, top, and [truss] ...
	if ($pid) {
		if ( system ("pstack $pid >$outfile.ps") ) { 
			print "error on attempt to run pstack command: $!\n";
		 }
		if ( system ("top >$outfile.top") ) { 
			print "error on attempt to run top command: $!\n";
		 }
		if ($doTruss) {
			if ($tmpid = fork) {
				print "waiting $tsec seconds to kill truss process ...\n";
				system ("sleep $tsec");
				unless (system("kill -9 $tmpid")) {
					print "truss process ($tmpid) killed\n";
				} else {
					print "unsuccessful attempt to kill truss process ($tmpid)\n";
				}				
			} else {
				exec ("truss -o $outfile.tr -p $pid") or die "Can't exec: $!\n";
			}
		}
	} else {
		print "unable to run pstack or truss - could not find RoseRT process\n";
	}
}
#
#-----------------------------------------------------------------------------
#
&main;
#
